// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 3, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, Inc., 51 Franklin Street, Fifth
// Floor, Boston, MA 02110-1301, USA.
#include "tup.h"

/**
 * Constructor of the main window.
 *
 * @param parent The parent QObject.
 */
Tup::Tup(QTranslator *translator, QWidget *parent) : QTabWidget(parent)
{
  QStringListModel *languagesModel = new QStringListModel();

  /// Initialising values
  m_version = QString("v0.0.0.90qt");
  m_translator = translator;
  m_proxyUsed = 0;
  m_proxyHostname = "";
  m_proxyPort = 0;
  m_proxyAuthUsed = 0;
  m_proxyUsername = "";
  m_proxyPassword = "";
  /// Generate GUI
  setupUi(this);
  connectUi(this);
  languagesModel->setStringList(InitLanguages());
  this->ChangeLanguageListView->setModel(languagesModel);
  m_machine = new TupMachine();
  if (!loadSettings())
    {
      UpdateButton->setEnabled(false);
      /// no settings found. Set update frequence to 30 minutes
      m_tupSendTimerId = startTimer(1800000);
    }
  // /// Translate GUI
  // retranslateUi(this);
  displayUptime();
  // displayVersion();
  displaySysInfo();
  m_tupPaintTimerId = startTimer(500);
  this->OtherBox->hide();
}

/**
 * Feeds the language list listview.
 *
 *
 * @return The list of available languages.
 */
QStringList Tup::InitLanguages()
{
  QList<QString> languagesList = QList<QString>();

  m_languageActionGroup = new QActionGroup(this);
  QDir qmDir = QApplication::applicationDirPath();
  QStringList fileNames = qmDir.entryList(QStringList("tup_*.qm"));
  for (int i = 0; i < fileNames.size(); ++i)
    {
      QString locale = fileNames[i];
      locale.remove(0, locale.indexOf('_') + 1);
      locale.chop(3);

      QTranslator translator;
      translator.load(fileNames[i], qmDir.absolutePath());
      QString language = translator.translate("Tup", "English");
      languagesList.append(language);
      QAction *action = new QAction(tr("&%1 %2").arg(i + 1)
				    .arg(language), this);
      action->setCheckable(true);
      action->setData(locale);
      m_languageActionGroup->addAction(action);
      if (language == "English")
	action->setChecked(true);
    }
  return QStringList(languagesList);
}

/**
 * Switches the language of the GUI.
 *
 */
void Tup::switchLanguage()
{
  int index = ChangeLanguageListView->currentIndex().row();
  QList<QAction*> actions = m_languageActionGroup->actions();
  QString locale = actions[index]->data().toString();
  QString qmPath = QApplication::applicationDirPath();

  m_translator->load("tup_" + locale, qmPath);
  retranslateUi(this);
  this->setWindowTitle("Tugs' Uptime Project - "		\
		       "By ThArGos " + this->getVersion());
}

/**
 * Builds connections between the GUI and functions.
 *
 * @param tupUi The TabWidget component.
 */
void Tup::connectUi(QTabWidget *tupUi)
{
  connect(SaveButton, SIGNAL(clicked()), tupUi, SLOT(saveSettingsToFile()));
  connect(UpdateButton, SIGNAL(clicked()), tupUi, SLOT(sendUptime()));
  connect(ChangeLanguageButton, SIGNAL(clicked()),
	  this, SLOT(switchLanguage()));
}

/**
 * Sends the uptime to the tup server
 *
 */
void Tup::sendUptime()
{
  TupSend *send = new TupSend(m_settings, m_machine);

  this->ConsoleTextEdit->append("Sending uptime");
  /// TODO: exception management
  /// TODO: update last and next update time on the gui.
  send->sendInformations();
  delete send;
}

/**
 * Saves the user settings.
 *
 */
void Tup::saveVariableSettings()
{
  /// Saving user settings
  m_username = UsernameEdit->text();
  m_password = PasswordEdit->text();
  m_updatefreq = (UpdateFreqComboBox->currentText()).toInt();
  m_updatefreqindex = UpdateFreqComboBox->currentIndex();
  m_machinename = MachineNameEdit->text();
  /// Relaunching the "send update" timer
  killTimer(m_tupSendTimerId);
  m_tupSendTimerId = 0;
  /// Don't forget to convert minutes to milliseconds
  m_tupSendTimerId = startTimer(m_updatefreq * 1000 * 60);
  /// Do we use proxy?
  if (UseProxyCheckBox->isChecked())
    {
      m_proxyUsed = 1;
      m_proxyHostname = ProxyServerLineEdit->text();
      m_proxyPort = ProxyPortLineEdit->text().toInt();
      /// Do we use authentication?
      if (UseAuthCheckBox->isChecked())
	{
	  m_proxyAuthUsed = 1;
	  m_proxyUsername = ProxyUserLineEdit->text();
	  m_proxyPassword = ProxyPasswordLineEdit->text();
	}
      else
	{
	  m_proxyAuthUsed = 0;
	}
    }
  else
    {
      m_proxyUsed = 0;
    }
  m_settings.insert("username", m_username);
  m_settings.insert("password", m_password);
  m_settings.insert("updatefreqindex", QString().setNum(m_updatefreqindex));
  m_settings.insert("machinename", m_machinename);
  /// Connection settings
  m_settings.insert("proxyused", QString().setNum(m_proxyUsed));
  m_settings.insert("proxyhostname", m_proxyHostname);
  m_settings.insert("proxyport", QString().setNum(m_proxyPort));
  m_settings.insert("proxyauthused", QString().setNum(m_proxyAuthUsed));
  m_settings.insert("proxyusername", m_proxyUsername);
  m_settings.insert("proxypassword", m_proxyPassword);
}

/**
 * Saves the user settings to a file
 *
 */
void Tup::saveSettingsToFile()
{
  QFile file("settings.tup");

  saveVariableSettings();
  if (!file.open(QIODevice::WriteOnly))
    {
      QMessageBox::information(this, tr("Unable to open file"),
			       file.errorString());
      return;
    }
  QDataStream outputStream(&file);
  outputStream.setVersion(QDataStream::Qt_4_5);
  outputStream << m_settings;
  UpdateButton->setEnabled(true);
}

/**
 * Loads the application settings
 *
 */
bool Tup::loadSettings()
{
  QFile file("settings.tup");
  bool result = false;

  if (!file.open(QIODevice::ReadOnly))
    {
      /// can't open file
      result = false;
    }
  else
    {
      /// open file
      QDataStream inputStream(&file);
      inputStream.setVersion(QDataStream::Qt_4_5);
      /// empty existing settings
      m_settings.empty();
      /// feed the settings object
      inputStream >> m_settings;
      /// we successfully loaded the settings object
      result = true;
      /// load the settings into the interface
      UsernameEdit->setText(m_settings.find("username").value());
      PasswordEdit->setText(m_settings.find("password").value());
      MachineNameEdit->setText(m_settings.find("machinename").value());
      m_updatefreqindex = m_settings.find("updatefreqindex").value().toInt();
      UpdateFreqComboBox->setCurrentIndex(m_updatefreqindex);
      m_updatefreq = UpdateFreqComboBox->currentText().toInt();
      /// proxy settings
      if (m_settings.find("proxyused").value().toInt())
	{ UseProxyCheckBox->setChecked(true); }
      else
	{ UseProxyCheckBox->setChecked(false); }
      ProxyServerLineEdit->setText(m_settings.find("proxyhostname").value());
      if (m_settings.find("proxyport") != m_settings.end())
	{ ProxyPortLineEdit->setText(m_settings.find("proxyport").value()); }
      else
	{ ProxyPortLineEdit->setText(0); }
      if (m_settings.find("proxyauthused").value().toInt())
	{ UseAuthCheckBox->setChecked(true); }
      else
	{ UseAuthCheckBox->setChecked(false); }
      ProxyUserLineEdit->setText(m_settings.find("proxyusername").value());
      ProxyPasswordLineEdit->setText(m_settings.find("proxypassword").value());
      /// end of proxy settings
      /// relaunching the "send update" timer
      killTimer(m_tupSendTimerId);
      m_tupSendTimerId = 0;
      /// don't forget to convert minutes to milliseconds
      m_tupSendTimerId = startTimer(m_updatefreq * 1000 * 60);
    }
  return result;
}

void Tup::displayUptime()
{
  double uptime = m_machine->getUptime();
  double seconds, minutes, hours, days; // , months, years;

  seconds = minutes = hours = days = 0;  // = months = years = 0;
  seconds = static_cast<int>(uptime) % 60;
  minutes = static_cast<int>(uptime / 60) % 60;
  hours = static_cast<int>(uptime / 3600) % 24;
  days = static_cast<int>(uptime / 86400) % 30;
  // months = static_cast<int>(uptime / 2635200) % 12;
  // years = static_cast<int>(uptime / 31622400);
  UptimeInfoUptimeLabelEdit->setText(tr("%n day(s) ", "", days) +
				     tr("%n hour(s) ", "", hours) +
				     tr("%n minute(s) ", "", minutes) +
				     tr("%n second(s)", "", seconds));
}

void Tup::displaySysInfo()
{
  UptimeInfoOSLabelEdit->setText(m_machine->getSysInfoDescription());
}

/**
 * Timer event. Used to refresh the uptime displayed on the main
 * window
 *
 * @param event The timer event.
 */
void Tup::timerEvent(QTimerEvent *event)
{
  if (event->timerId() == m_tupPaintTimerId)
    {
      displayUptime();
    }
  else if (event->timerId() == m_tupSendTimerId)
    {
      sendUptime();
    }
  else
    {
      QWidget::timerEvent(event);
    }
}

/**
 * Used to kill the tup timer event for the paint event
 *
 */
void Tup::hideEvent(QHideEvent * /* event */)
{
  killTimer(m_tupPaintTimerId);
  m_tupPaintTimerId = 0;
}

/**
 * Used to set up the tup timer event for the paint event
 *
 */
void Tup::showEvent(QShowEvent * /* event */)
{
  if (m_tupPaintTimerId == 0)
    {
      m_tupPaintTimerId = startTimer(500);
    }
}

QString Tup::getVersion()
{
  return m_version;
}
